@@ -135,12 +135,72 @@ where
         multispace0,
         alt((
             map(parse_primitive_type, Type::Primitive),
-            parse_unrsolved_path,
+            parse_unresolved_path,
+            parse_tuple,
+            parse_slice,
+            value(Type::Never, char('!')),
+            parse_raw_pointer,
+            parse_borrowed_ref,
         )),
     )(i)
 }
 
-fn parse_unrsolved_path<'a, E>(i: &'a str) -> IResult<&'a str, Type, E>
+fn parse_tuple<'a, E>(i: &'a str) -> IResult<&'a str, Type, E>
+where
+    E: ParseError<&'a str> + ContextError<&'a str>,
+{
+    map(
+        delimited(
+            char('('),
+            separated_list0(char(','), preceded(multispace0, parse_type)),
+            char(')'),
+        ),
+        Type::Tuple,
+    )(i)
+}
+
+fn parse_slice<'a, E>(i: &'a str) -> IResult<&'a str, Type, E>
+where
+    E: ParseError<&'a str> + ContextError<&'a str>,
+{
+    map(delimited(char('['), parse_type, char(']')), |ty| {
+        Type::Slice(Box::new(ty))
+    })(i)
+}
+
+fn parse_raw_pointer<'a, E>(i: &'a str) -> IResult<&'a str, Type, E>
+where
+    E: ParseError<&'a str> + ContextError<&'a str>,
+{
+    let (i, mutable) = alt((map(tag("*mut"), |_| true), map(tag("*const"), |_| false)))(i)?;
+    let (i, type_) = parse_type(i)?;
+
+    Ok((
+        i,
+        Type::RawPointer {
+            mutable,
+            type_: Box::new(type_),
+        },
+    ))
+}
+
+fn parse_borrowed_ref<'a, E>(i: &'a str) -> IResult<&'a str, Type, E>
+where
+    E: ParseError<&'a str> + ContextError<&'a str>,
+{
+    let (i, mutable) = alt((map(tag("&mut"), |_| true), map(tag("&"), |_| false)))(i)?;
+    let (i, type_) = parse_type(i)?;
+
+    Ok((
+        i,
+        Type::BorrowedRef {
+            mutable,
+            type_: Box::new(type_),
+        },
+    ))
+}
+
+fn parse_unresolved_path<'a, E>(i: &'a str) -> IResult<&'a str, Type, E>
 where
     E: ParseError<&'a str> + ContextError<&'a str>,
 {
